@@ -55,7 +55,8 @@ from salt.version import (
     )
 # is there not SaltStackVersion.current() to get
 # the version of the salt running this code??
-CUR_VER = SaltStackVersion(__version__[0], __version__[1])
+_version_ary = __version__.split('.')
+CUR_VER = SaltStackVersion(_version_ary[0], _version_ary[1])
 BORON = SaltStackVersion.from_name('Boron')
 
 # pylint: disable=import-error
@@ -127,7 +128,7 @@ def _auth(profile=None, api_version=2, **connection_args):
     admin_token = get('token')
     region = get('region')
     ks_endpoint = get('endpoint', 'http://127.0.0.1:9292/')
-    g_endpoint_url = __salt__['keystone.endpoint_get']('glance')
+    g_endpoint_url = __salt__['keystone.endpoint_get']('glance', profile)
     # The trailing 'v2' causes URLs like thise one:
     # http://127.0.0.1:9292/v2/v1/images
     g_endpoint_url = re.sub('/v2', '', g_endpoint_url['internalurl'])
@@ -293,7 +294,7 @@ def image_create(name, location=None, profile=None, visibility=None,
     # in a usable fashion. Thus we have to use v1 for now.
     g_client = _auth(profile, api_version=1)
     image = g_client.images.create(name=name, **kwargs)
-    return image_show(image.id)
+    return image_show(image.id, profile=profile)
 
 
 def image_delete(id=None, name=None, profile=None):  # pylint: disable=C0103
@@ -460,13 +461,13 @@ def image_update(id=None, name=None, profile=None, **kwargs):  # pylint: disable
     - visibility ('public' or 'private')
     '''
     if id:
-        image = image_show(id=id)
+        image = image_show(id=id, profile=profile)
         if 'result' in image and not image['result']:
             return image
         elif len(image) == 1:
             image = image.values()[0]
     elif name:
-        img_list = image_list(name=name)
+        img_list = image_list(name=name, profile=profile)
         if img_list is dict and 'result' in img_list:
             return img_list
         elif len(img_list) == 0:
