@@ -11,6 +11,8 @@ import os
 import re
 import fnmatch
 import collections
+import time
+import datetime
 
 # Import 3rd-party libs
 import salt.ext.six as six
@@ -23,16 +25,18 @@ import salt.utils.event
 from salt.utils.network import host_to_ip as _host_to_ip
 from salt.utils.network import remote_port_tcp as _remote_port_tcp
 from salt.ext.six.moves import zip
+from salt.utils.decorators import with_deprecated
+from salt.exceptions import CommandExecutionError
 
-
+__virtualname__ = 'status'
 __opts__ = {}
 
 
-# TODO: Make this module support windows hosts
 def __virtual__():
     if salt.utils.is_windows():
-        return False
-    return True
+        return False, 'Windows platform is not supported by this module'
+
+    return __virtualname__
 
 
 def _number(text):
@@ -121,9 +125,10 @@ def custom():
     return ret
 
 
+@with_deprecated(globals(), "Boron")
 def uptime():
     '''
-    Return the uptime for this minion
+    Return the uptime for this system.
 
     CLI Example:
 
@@ -131,7 +136,52 @@ def uptime():
 
         salt '*' status.uptime
     '''
-    return __salt__['cmd.run']('uptime')
+    ut_path = "/proc/uptime"
+    if not os.path.exists(ut_path):
+        raise CommandExecutionError("File {ut_path} was not found.".format(ut_path=ut_path))
+
+    ut_ret = {
+        'seconds': int(float(open(ut_path).read().strip().split()[0]))
+    }
+
+    utc_time = datetime.datetime.utcfromtimestamp(time.time() - ut_ret['seconds'])
+    ut_ret['since_iso'] = utc_time.isoformat()
+    ut_ret['since_t'] = time.mktime(utc_time.timetuple())
+    ut_ret['days'] = ut_ret['seconds'] / 60 / 60 / 24
+    hours = (ut_ret['seconds'] - (ut_ret['days'] * 24 * 60 * 60)) / 60 / 60
+    minutes = ((ut_ret['seconds'] - (ut_ret['days'] * 24 * 60 * 60)) / 60) - hours * 60
+    ut_ret['time'] = '{0}:{1}'.format(hours, minutes)
+    ut_ret['users'] = len(__salt__['cmd.run']("who -s").split(os.linesep))
+
+    return ut_ret
+
+
+def _uptime(human_readable=True):
+    '''
+    Return the uptime for this minion
+
+    human_readable: True
+        If ``True`` return the output provided by the system.  If ``False``
+        return the output in seconds.
+
+        .. versionadded:: 2015.8.4
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' status.uptime
+    '''
+    if human_readable:
+        return __salt__['cmd.run']('uptime')
+    else:
+        if os.path.exists('/proc/uptime'):
+            out = __salt__['cmd.run']('cat /proc/uptime').split()
+            if len(out):
+                return out[0]
+            else:
+                return 'unexpected format in /proc/uptime'
+        return 'cannot find /proc/uptime'
 
 
 def loadavg():
