@@ -569,7 +569,7 @@ def assertion(assertion):
 
     .. code-block:: bash
 
-        salt '*' test.assert False
+        salt '*' test.assertion False
     '''
     assert assertion
 
