@@ -118,24 +118,24 @@ The token may be sent in one of two ways:
 
   For example, using curl:
 
-    .. code-block:: bash
+  .. code-block:: bash
 
-        curl -sSk https://localhost:8000/login \\
-            -H 'Accept: application/x-yaml' \\
-            -d username=saltdev \\
-            -d password=saltdev \\
-            -d eauth=auto
+      curl -sSk https://localhost:8000/login \\
+          -H 'Accept: application/x-yaml' \\
+          -d username=saltdev \\
+          -d password=saltdev \\
+          -d eauth=auto
 
-Copy the ``token`` value from the output and include it in subsequent requests:
+  Copy the ``token`` value from the output and include it in subsequent requests:
 
-    .. code-block:: bash
+  .. code-block:: bash
 
-        curl -sSk https://localhost:8000 \\
-            -H 'Accept: application/x-yaml' \\
-            -H 'X-Auth-Token: 697adbdc8fe971d09ae4c2a3add7248859c87079'\\
-            -d client=local \\
-            -d tgt='*' \\
-            -d fun=test.ping
+      curl -sSk https://localhost:8000 \\
+          -H 'Accept: application/x-yaml' \\
+          -H 'X-Auth-Token: 697adbdc8fe971d09ae4c2a3add7248859c87079'\\
+          -d client=local \\
+          -d tgt='*' \\
+          -d fun=test.ping
 
 * Sent via a cookie. This option is a convenience for HTTP clients that
   automatically handle cookie support (such as browsers).
@@ -680,7 +680,7 @@ def lowdata_fmt():
     # if the data was sent as urlencoded, we need to make it a list.
     # this is a very forgiving implementation as different clients set different
     # headers for form encoded data (including charset or something similar)
-    if not isinstance(data, list):
+    if data and not isinstance(data, list):
         # Make the 'arg' param a list if not already
         if 'arg' in data and not isinstance(data['arg'], list):
             data['arg'] = [data['arg']]
@@ -699,7 +699,7 @@ cherrypy.tools.salt_auth = cherrypy.Tool('before_request_body',
         salt_auth_tool, priority=60)
 cherrypy.tools.hypermedia_in = cherrypy.Tool('before_request_body',
         hypermedia_in)
-cherrypy.tools.cors_tool = cherrypy.Tool('before_handler',
+cherrypy.tools.cors_tool = cherrypy.Tool('before_request_body',
         cors_tool, priority=30)
 cherrypy.tools.lowdata_fmt = cherrypy.Tool('before_handler',
         lowdata_fmt, priority=40)
@@ -758,6 +758,10 @@ class LowDataAdapter(object):
         for chunk in lowstate:
             if token:
                 chunk['token'] = token
+                if cherrypy.session.get('user'):
+                    chunk['__current_eauth_user'] = cherrypy.session.get('user')
+                if cherrypy.session.get('groups'):
+                    chunk['__current_eauth_groups'] = cherrypy.session.get('groups')
 
             if client:
                 chunk['client'] = client
@@ -1180,6 +1184,7 @@ class Keys(LowDataAdapter):
     module <salt.wheel.key>` functions.
     '''
 
+    @cherrypy.config(**{'tools.salt_token.on': True})
     def GET(self, mid=None):
         '''
         Show the list of minion keys or detail on a specific key
@@ -1247,8 +1252,6 @@ class Keys(LowDataAdapter):
               minions:
                 jerry: 51:93:b3:d0:9f:3a:6d:e5:28:67:c2:4b:27:d6:cd:2b
         '''
-        self._cp_config['tools.salt_token.on'] = True
-
         if mid:
             lowstate = [{
                 'client': 'wheel',
@@ -1266,11 +1269,13 @@ class Keys(LowDataAdapter):
 
         return {'return': next(result, {}).get('data', {}).get('return', {})}
 
-    def POST(self, mid, keysize=None, force=None, **kwargs):
+    @cherrypy.config(**{'tools.hypermedia_out.on': False, 'tools.sessions.on': False})
+    def POST(self, **kwargs):
         r'''
         Easily generate keys for a minion and auto-accept the new key
 
-        .. versionadded:: 2014.7.0
+        Accepts all the same parameters as the :py:func:`key.gen_accept
+        <salt.wheel.key.gen_accept>`.
 
         Example partial kickstart script to bootstrap a new minion:
 
@@ -1326,24 +1331,15 @@ class Keys(LowDataAdapter):
 
             jerry.pub0000644000000000000000000000070300000000000010730 0ustar  00000000000000
         '''
-        self._cp_config['tools.hypermedia_out.on'] = False
-        self._cp_config['tools.sessions.on'] = False
-
-        lowstate = [{
+        lowstate = cherrypy.request.lowstate
+        lowstate[0].update({
             'client': 'wheel',
             'fun': 'key.gen_accept',
-            'id_': mid,
-        }]
+        })
 
-        if keysize:
-            lowstate[0]['keysize'] = keysize
+        if 'mid' in lowstate[0]:
+            lowstate[0]['id_'] = lowstate[0].pop('mid')
 
-        if force:
-            lowstate[0]['force'] = force
-
-        lowstate[0].update(kwargs)
-
-        cherrypy.request.lowstate = lowstate
         result = self.exec_lowstate()
         ret = next(result, {}).get('data', {}).get('return', {})
 
@@ -1362,7 +1358,7 @@ class Keys(LowDataAdapter):
         tarball.close()
 
         headers = cherrypy.response.headers
-        headers['Content-Disposition'] = 'attachment; filename="saltkeys-{0}.tar"'.format(mid)
+        headers['Content-Disposition'] = 'attachment; filename="saltkeys-{0}.tar"'.format(lowstate[0]['id_'])
         headers['Content-Type'] = 'application/x-tar'
         headers['Content-Length'] = fileobj.len
         headers['Cache-Control'] = 'no-cache'
@@ -1507,6 +1503,9 @@ class Login(LowDataAdapter):
         cherrypy.response.headers['X-Auth-Token'] = cherrypy.session.id
         cherrypy.session['token'] = token['token']
         cherrypy.session['timeout'] = (token['expire'] - token['start']) / 60
+        cherrypy.session['user'] = token['name']
+        if 'groups' in token:
+            cherrypy.session['groups'] = token['groups']
 
         # Grab eauth config for the current backend for the current user
         try:
@@ -1636,7 +1635,7 @@ class Run(LowDataAdapter):
 
         All SSH client requests are synchronous.
 
-        ** Example SSH client request:**
+        **Example SSH client request:**
 
         .. code-block:: bash
 
@@ -1685,6 +1684,7 @@ class Events(object):
     Salt infrastructure.
 
     .. seealso:: :ref:`events`
+
     '''
     exposed = True
 
@@ -1924,7 +1924,7 @@ class WebsocketEndpoint(object):
         :status 401: |401|
         :status 406: |406|
 
-        **Example request:**
+        **Example request:** ::
 
             curl -NsSk \\
                 -H 'X-Auth-Token: ffedf49d' \\
@@ -2212,7 +2212,9 @@ class Webhook(object):
         '''
         tag = '/'.join(itertools.chain(self.tag_base, args))
         data = cherrypy.serving.request.unserialized_data
-        raw_body = cherrypy.serving.request.raw_body
+        if not data:
+            data = {}
+        raw_body = getattr(cherrypy.serving.request, 'raw_body', '')
         headers = dict(cherrypy.request.headers)
 
         ret = self.event.fire_event({
