@@ -87,6 +87,30 @@ def _format_comments(comments):
     return ret
 
 
+def _map_port_from_yaml_to_docker(port):
+    '''
+    docker-py interface is not very nice:
+    While for ``port_bindings`` they support:
+
+    .. code-block:: python
+
+        '8888/tcp'
+
+    For ``ports``, it has to be transformed into:
+
+    .. code-block:: python
+
+        (8888, 'tcp')
+
+    '''
+    if isinstance(port, six.string_types):
+        port, sep, protocol = port.partition('/')
+        if protocol:
+            return int(port), protocol
+        return int(port)
+    return port
+
+
 def _prep_input(kwargs):
     '''
     Repack (if necessary) data that should be in a dict but is easier to
@@ -197,7 +221,7 @@ def _compare(actual, create_kwargs, defaults_from_image):
             for port_def in data:
                 if isinstance(port_def, six.integer_types):
                     port_def = str(port_def)
-                if isinstance(port_def, tuple):
+                if isinstance(port_def, (tuple, list)):
                     desired_ports.append('{0}/{1}'.format(*port_def))
                 elif '/' not in port_def:
                     desired_ports.append('{0}/tcp'.format(port_def))
@@ -387,6 +411,21 @@ def _compare(actual, create_kwargs, defaults_from_image):
             # sometimes `[]`. We have to deal with it.
             if bool(actual_data) != bool(data):
                 ret.update({item: {'old': actual_data, 'new': data}})
+        elif item == 'labels':
+            if actual_data is None:
+                actual_data = {}
+            if data is None:
+                data = {}
+            image_labels = _image_get(config['image_path'], default={})
+            if image_labels is not None:
+                image_labels = image_labels.copy()
+                if isinstance(data, list):
+                    data = dict((k, '') for k in data)
+                image_labels.update(data)
+                data = image_labels
+            if actual_data != data:
+                ret.update({item: {'old': actual_data, 'new': data}})
+                continue
 
         elif isinstance(data, list):
             # Compare two sorted lists of items. Won't work for "command"
@@ -1534,7 +1573,8 @@ def running(name,
         if create_kwargs.get('port_bindings') is not None:
             # Be smart and try to provide `ports` argument derived from
             # the "port_bindings" configuration.
-            auto_ports = list(create_kwargs['port_bindings'])
+            auto_ports = [_map_port_from_yaml_to_docker(port)
+                          for port in create_kwargs['port_bindings']]
             actual_ports = create_kwargs.setdefault('ports', [])
             actual_ports.extend([p for p in auto_ports if
                                  p not in actual_ports])
@@ -2201,7 +2241,9 @@ def volume_present(name, driver=None, driver_opts=None, force=False):
            'comment': ''}
     if salt.utils.is_dictlist(driver_opts):
         driver_opts = salt.utils.repack_dictlist(driver_opts)
-    volumes = [v for v in __salt__['dockerng.volumes']()['Volumes'] if v['Name'] == name]
+    volumes = __salt__['dockerng.volumes']()['Volumes']
+    if volumes is not None:
+        volumes = [v for v in volumes if v['Name'] == name]
     if not volumes:
         try:
             ret['changes']['created'] = __salt__['dockerng.create_volume'](
