@@ -145,16 +145,18 @@ def _get_comparison_spec(pkgver):
     return oper, verstr
 
 
-def _fulfills_version_spec(versions, oper, desired_version):
+def _fulfills_version_spec(versions, oper, desired_version,
+                           ignore_epoch=False):
     '''
     Returns True if any of the installed versions match the specified version,
     otherwise returns False
     '''
+    normalize = lambda x: x.split(':', 1)[-1] if ignore_epoch else x
     cmp_func = __salt__.get('pkg.version_cmp')
     for ver in versions:
-        if salt.utils.compare_versions(ver1=ver,
+        if salt.utils.compare_versions(ver1=normalize(ver),
                                        oper=oper,
-                                       ver2=desired_version,
+                                       ver2=normalize(desired_version),
                                        cmp_func=cmp_func):
             return True
     return False
@@ -177,6 +179,7 @@ def _find_remove_targets(name=None,
                          version=None,
                          pkgs=None,
                          normalize=True,
+                         ignore_epoch=False,
                          **kwargs):
     '''
     Inspect the arguments to pkg.removed and discover what packages need to
@@ -227,7 +230,8 @@ def _find_remove_targets(name=None,
         except CommandExecutionError as exc:
             problems.append(exc.strerror)
             continue
-        if not _fulfills_version_spec(cver, oper, verstr):
+        if not _fulfills_version_spec(cver, oper, verstr,
+                                      ignore_epoch=ignore_epoch):
             log.debug(
                 'Current version ({0}) did not match desired version '
                 'specification ({1}), will not remove'
@@ -263,6 +267,7 @@ def _find_install_targets(name=None,
                           skip_suggestions=False,
                           pkg_verify=False,
                           normalize=True,
+                          ignore_epoch=False,
                           **kwargs):
     '''
     Inspect the arguments to pkg.installed and discover what packages need to
@@ -368,27 +373,35 @@ def _find_install_targets(name=None,
                 for name, version in desired.items()
                 if not (name in cur_pkgs and version in (None, cur_pkgs[name]))
             ])
-            problems = _preflight_check(not_installed, **kwargs)
-            comments = []
-            if problems.get('no_suggest'):
-                comments.append(
-                    'The following package(s) were not found, and no possible '
-                    'matches were found in the package db: '
-                    '{0}'.format(', '.join(sorted(problems['no_suggest'])))
-                )
-            if problems.get('suggest'):
-                for pkgname, suggestions in six.iteritems(problems['suggest']):
-                    comments.append(
-                        'Package \'{0}\' not found (possible matches: {1})'
-                        .format(pkgname, ', '.join(suggestions))
-                    )
-            if comments:
-                if len(comments) > 1:
-                    comments.append('')
-                return {'name': name,
-                        'changes': {},
-                        'result': False,
-                        'comment': '. '.join(comments).rstrip()}
+            if not_installed:
+                try:
+                    problems = _preflight_check(not_installed, **kwargs)
+                except CommandExecutionError:
+                    pass
+                else:
+                    comments = []
+                    if problems.get('no_suggest'):
+                        comments.append(
+                            'The following package(s) were not found, and no '
+                            'possible matches were found in the package db: '
+                            '{0}'.format(
+                                ', '.join(sorted(problems['no_suggest']))
+                            )
+                        )
+                    if problems.get('suggest'):
+                        for pkgname, suggestions in \
+                                six.iteritems(problems['suggest']):
+                            comments.append(
+                                'Package \'{0}\' not found (possible matches: '
+                                '{1})'.format(pkgname, ', '.join(suggestions))
+                            )
+                    if comments:
+                        if len(comments) > 1:
+                            comments.append('')
+                        return {'name': name,
+                                'changes': {},
+                                'result': False,
+                                'comment': '. '.join(comments).rstrip()}
 
     # Find out which packages will be targeted in the call to pkg.install
     targets = {}
@@ -454,7 +467,8 @@ def _find_install_targets(name=None,
         if not sources and 'allow_updates' in kwargs:
             if kwargs['allow_updates']:
                 oper = '>='
-        if not _fulfills_version_spec(cver, oper, verstr):
+        if not _fulfills_version_spec(cver, oper, verstr,
+                                      ignore_epoch=ignore_epoch):
             log.debug(
                 'Current version ({0}) did not match desired version '
                 'specification ({1}), adding to installation targets'
@@ -489,7 +503,7 @@ def _find_install_targets(name=None,
     return desired, targets, to_unpurge, to_reinstall, altered_files, warnings
 
 
-def _verify_install(desired, new_pkgs):
+def _verify_install(desired, new_pkgs, ignore_epoch=False):
     '''
     Determine whether or not the installed packages match what was requested in
     the SLS file.
@@ -502,6 +516,8 @@ def _verify_install(desired, new_pkgs):
 
         if __grains__['os'] == 'FreeBSD' and origin:
             cver = [k for k, v in six.iteritems(new_pkgs) if v['origin'] == pkgname]
+        elif __grains__['os_family'] == 'Debian':
+            cver = new_pkgs.get(pkgname.split('=')[0])
         else:
             cver = new_pkgs.get(pkgname)
 
@@ -515,7 +531,8 @@ def _verify_install(desired, new_pkgs):
             ok.append(pkgname)
             continue
         oper, verstr = _get_comparison_spec(pkgver)
-        if _fulfills_version_spec(cver, oper, verstr):
+        if _fulfills_version_spec(cver, oper, verstr,
+                                  ignore_epoch=ignore_epoch):
             ok.append(pkgname)
         else:
             failed.append(pkgname)
@@ -576,6 +593,7 @@ def installed(
         allow_updates=False,
         pkg_verify=False,
         normalize=True,
+        ignore_epoch=False,
         **kwargs):
     '''
     Ensure that the package is installed, and that it is the correct version
@@ -590,7 +608,7 @@ def installed(
 
     :param str version:
         Install a specific version of a package. This option is ignored if
-        either "pkgs" or "sources" is used. Currently, this option is supported
+        "sources" is used. Currently, this option is supported
         for the following pkg providers: :mod:`apt <salt.modules.aptpkg>`,
         :mod:`ebuild <salt.modules.ebuild>`,
         :mod:`pacman <salt.modules.pacman>`,
@@ -603,9 +621,29 @@ def installed(
 
         .. code-block:: bash
 
-            # salt myminion pkg.latest_version httpd
+            # salt myminion pkg.latest_version vim-enhanced
             myminion:
-                2.2.15-30.el6.centos
+                2:7.4.160-1.el7
+
+        .. important::
+            As of version 2015.8.7, for distros which use yum/dnf, packages
+            which have a version with a nonzero epoch (that is, versions which
+            start with a number followed by a colon like in the
+            ``pkg.latest_version`` output above) must have the epoch included
+            when specifying the version number. For example:
+
+            .. code-block:: yaml
+
+                vim-enhanced:
+                  pkg.installed:
+                    - version: 2:7.4.160-1.el7
+
+            In version 2015.8.9, an **ignore_epoch** argument has been added to
+            :py:mod:`pkg.installed <salt.states.pkg.installed>`,
+            :py:mod:`pkg.removed <salt.states.pkg.installed>`, and
+            :py:mod:`pkg.purged <salt.states.pkg.installed>` states, which
+            causes the epoch to be disregarded when the state checks to see if
+            the desired version was installed.
 
         Also, while this function is not yet implemented for all pkg frontends,
         :mod:`pkg.list_repo_pkgs <salt.modules.yumpkg.list_repo_pkgs>` will
@@ -631,6 +669,18 @@ def installed(
         The version strings returned by either of these functions can be used
         as version specifiers in pkg states.
 
+        You can install a specific version when using the ``pkgs`` argument by
+        including the version after the package:
+
+        .. code-block:: yaml
+
+            common_packages:
+              pkg.installed:
+                - pkgs:
+                  - unzip
+                  - dos2unix
+                  - salt-minion: 2015.8.5-1.el6
+
     :param bool refresh:
         Update the repo database of available packages prior to installing the
         requested package.
@@ -697,92 +747,6 @@ def installed(
 
         .. versionadded:: 2014.1.1
 
-    :param list pkgs:
-        A list of packages to install from a software repository. All packages
-        listed under ``pkgs`` will be installed via a single command.
-
-        Example:
-
-        .. code-block:: yaml
-
-            mypkgs:
-              pkg.installed:
-                - pkgs:
-                  - foo
-                  - bar
-                  - baz
-                - hold: True
-
-        ``NOTE:`` For :mod:`apt <salt.modules.aptpkg>`,
-        :mod:`ebuild <salt.modules.ebuild>`,
-        :mod:`pacman <salt.modules.pacman>`, :mod:`yumpkg <salt.modules.yumpkg>`,
-        and :mod:`zypper <salt.modules.zypper>`, version numbers can be specified
-        in the ``pkgs`` argument. For example:
-
-        .. code-block:: yaml
-
-            mypkgs:
-              pkg.installed:
-                - pkgs:
-                  - foo
-                  - bar: 1.2.3-4
-                  - baz
-
-        Additionally, :mod:`ebuild <salt.modules.ebuild>`,
-        :mod:`pacman <salt.modules.pacman>` and
-        :mod:`zypper <salt.modules.zypper>` support the ``<``, ``<=``, ``>=``, and
-        ``>`` operators for more control over what versions will be installed. For
-
-        Example:
-
-        .. code-block:: yaml
-
-            mypkgs:
-              pkg.installed:
-                - pkgs:
-                  - foo
-                  - bar: '>=1.2.3-4'
-                  - baz
-
-        ``NOTE:`` When using comparison operators, the expression must be enclosed
-        in quotes to avoid a YAML render error.
-
-        With :mod:`ebuild <salt.modules.ebuild>` is also possible to specify a
-        use flag list and/or if the given packages should be in
-        package.accept_keywords file and/or the overlay from which you want the
-        package to be installed.
-
-        For example:
-
-        .. code-block:: yaml
-
-            mypkgs:
-              pkg.installed:
-                - pkgs:
-                  - foo: '~'
-                  - bar: '~>=1.2:slot::overlay[use,-otheruse]'
-                  - baz
-
-        **Multiple Package Installation Options: (not supported in Windows or
-        pkgng)**
-
-    :param list sources:
-        A list of packages to install, along with the source URI or local path
-        from which to install each package. In the example below, ``foo``,
-        ``bar``, ``baz``, etc. refer to the name of the package, as it would
-        appear in the output of the ``pkg.version`` or ``pkg.list_pkgs`` salt
-        CLI commands.
-
-        .. code-block:: yaml
-
-            mypkgs:
-              pkg.installed:
-                - sources:
-                  - foo: salt://rpms/foo.rpm
-                  - bar: http://somesite.org/bar.rpm
-                  - baz: ftp://someothersite.org/baz.rpm
-                  - qux: /minion/path/to/qux.rpm
-
     :param bool allow_updates:
         Allow the package to be updated outside Salt's control (e.g. auto
         updates on Windows). This means a package on the Minion can have a
@@ -855,52 +819,169 @@ def installed(
               pkg.installed:
                 - normalize: False
 
-    :param kwargs:
-        These are specific to each OS. If it does not apply to the execution
-        module for your OS, it is ignored.
+    :param bool ignore_epoch:
+        When a package version contains an non-zero epoch (e.g.
+        ``1:3.14.159-2.el7``, and a specific version of a package is desired,
+        set this option to ``True`` to ignore the epoch when comparing
+        versions. This allows for the following SLS to be used:
 
-        :param bool hold:
-            Force the package to be held at the current installed version.
-            Currently works with YUM & APT based systems.
+        .. code-block:: yaml
 
-            .. versionadded:: 2014.7.0
+            # Actual vim-enhanced version: 2:7.4.160-1.el7
+            vim-enhanced:
+              pkg.installed:
+                - version: 7.4.160-1.el7
+                - ignore_epoch: True
 
-        :param list names:
-            A list of packages to install from a software repository. Each package
-            will be installed individually by the package manager.
+        Without this option set to ``True`` in the above example, the package
+        would be installed, but the state would report as failed because the
+        actual installed version would be ``2:7.4.160-1.el7``. Alternatively,
+        this option can be left as ``False`` and the full version string (with
+        epoch) can be specified in the SLS file:
 
-            .. warning::
+        .. code-block:: yaml
 
-                Unlike ``pkgs``, the ``names`` parameter cannot specify a version.
-                In addition, it makes a separate call to the package management
-                frontend to install each package, whereas ``pkgs`` makes just a
-                single call. It is therefore recommended to use ``pkgs`` instead of
-                ``names`` to install multiple packages, both for the additional
-                features and the performance improvement that it brings.
+            vim-enhanced:
+              pkg.installed:
+                - version: 2:7.4.160-1.el7
 
-        :param bool install_recommends:
-            Whether to install the packages marked as recommended. Default is
-            ``True``. Currently only works with APT-based systems.
+        .. versionadded:: 2015.8.9
 
-            .. versionadded:: 2015.5.0
+    |
 
-            .. code-block:: yaml
+    **MULTIPLE PACKAGE INSTALLATION OPTIONS: (not supported in Windows or
+    pkgng)**
 
-                httpd:
-                  pkg.installed:
-                    - install_recommends: False
+    :param list pkgs:
+        A list of packages to install from a software repository. All packages
+        listed under ``pkgs`` will be installed via a single command.
 
-        :param bool only_upgrade:
-            Only upgrade the packages, if they are already installed. Default is
-            ``False``. Currently only works with APT-based systems.
+        Example:
 
-            .. versionadded:: 2015.5.0
+        .. code-block:: yaml
 
-            .. code-block:: yaml
+            mypkgs:
+              pkg.installed:
+                - pkgs:
+                  - foo
+                  - bar
+                  - baz
+                - hold: True
 
-                httpd:
-                  pkg.installed:
-                    - only_upgrade: True
+        ``NOTE:`` For :mod:`apt <salt.modules.aptpkg>`,
+        :mod:`ebuild <salt.modules.ebuild>`,
+        :mod:`pacman <salt.modules.pacman>`, :mod:`yumpkg <salt.modules.yumpkg>`,
+        and :mod:`zypper <salt.modules.zypper>`, version numbers can be specified
+        in the ``pkgs`` argument. For example:
+
+        .. code-block:: yaml
+
+            mypkgs:
+              pkg.installed:
+                - pkgs:
+                  - foo
+                  - bar: 1.2.3-4
+                  - baz
+
+        Additionally, :mod:`ebuild <salt.modules.ebuild>`,
+        :mod:`pacman <salt.modules.pacman>` and
+        :mod:`zypper <salt.modules.zypper>` support the ``<``, ``<=``, ``>=``, and
+        ``>`` operators for more control over what versions will be installed. For
+
+        Example:
+
+        .. code-block:: yaml
+
+            mypkgs:
+              pkg.installed:
+                - pkgs:
+                  - foo
+                  - bar: '>=1.2.3-4'
+                  - baz
+
+        ``NOTE:`` When using comparison operators, the expression must be enclosed
+        in quotes to avoid a YAML render error.
+
+        With :mod:`ebuild <salt.modules.ebuild>` is also possible to specify a
+        use flag list and/or if the given packages should be in
+        package.accept_keywords file and/or the overlay from which you want the
+        package to be installed.
+
+        For example:
+
+        .. code-block:: yaml
+
+            mypkgs:
+              pkg.installed:
+                - pkgs:
+                  - foo: '~'
+                  - bar: '~>=1.2:slot::overlay[use,-otheruse]'
+                  - baz
+
+    :param list sources:
+        A list of packages to install, along with the source URI or local path
+        from which to install each package. In the example below, ``foo``,
+        ``bar``, ``baz``, etc. refer to the name of the package, as it would
+        appear in the output of the ``pkg.version`` or ``pkg.list_pkgs`` salt
+        CLI commands.
+
+        .. code-block:: yaml
+
+            mypkgs:
+              pkg.installed:
+                - sources:
+                  - foo: salt://rpms/foo.rpm
+                  - bar: http://somesite.org/bar.rpm
+                  - baz: ftp://someothersite.org/baz.rpm
+                  - qux: /minion/path/to/qux.rpm
+
+    **PLATFORM-SPECIFIC ARGUMENTS**
+
+    These are specific to each OS. If it does not apply to the execution
+    module for your OS, it is ignored.
+
+    :param bool hold:
+        Force the package to be held at the current installed version.
+        Currently works with YUM & APT based systems.
+
+        .. versionadded:: 2014.7.0
+
+    :param list names:
+        A list of packages to install from a software repository. Each package
+        will be installed individually by the package manager.
+
+        .. warning::
+
+            Unlike ``pkgs``, the ``names`` parameter cannot specify a version.
+            In addition, it makes a separate call to the package management
+            frontend to install each package, whereas ``pkgs`` makes just a
+            single call. It is therefore recommended to use ``pkgs`` instead of
+            ``names`` to install multiple packages, both for the additional
+            features and the performance improvement that it brings.
+
+    :param bool install_recommends:
+        Whether to install the packages marked as recommended. Default is
+        ``True``. Currently only works with APT-based systems.
+
+        .. versionadded:: 2015.5.0
+
+        .. code-block:: yaml
+
+            httpd:
+              pkg.installed:
+                - install_recommends: False
+
+    :param bool only_upgrade:
+        Only upgrade the packages, if they are already installed. Default is
+        ``False``. Currently only works with APT-based systems.
+
+        .. versionadded:: 2015.5.0
+
+        .. code-block:: yaml
+
+            httpd:
+              pkg.installed:
+                - only_upgrade: True
 
     :return:
         A dictionary containing the state of the software installation
@@ -933,6 +1014,10 @@ def installed(
 
     if version is not None and version == 'latest':
         version = __salt__['pkg.latest_version'](name)
+        # If version is empty, it means the latest version is installed
+        # so we grab that version to avoid passing an empty string
+        if not version:
+            version = __salt__['pkg.version'](name)
 
     kwargs['allow_updates'] = allow_updates
     result = _find_install_targets(name, version, pkgs, sources,
@@ -940,6 +1025,7 @@ def installed(
                                    skip_suggestions=skip_suggestions,
                                    pkg_verify=pkg_verify,
                                    normalize=normalize,
+                                   ignore_epoch=ignore_epoch,
                                    **kwargs)
 
     try:
@@ -1146,12 +1232,9 @@ def installed(
     else:
         if __grains__['os'] == 'FreeBSD':
             kwargs['with_origin'] = True
-        ok, failed = \
-            _verify_install(
-                desired, __salt__['pkg.list_pkgs'](
-                    versions_as_list=True, **kwargs
-                )
-            )
+        new_pkgs = __salt__['pkg.list_pkgs'](versions_as_list=True, **kwargs)
+        ok, failed = _verify_install(desired, new_pkgs,
+                                     ignore_epoch=ignore_epoch)
         modified = [x for x in ok if x in targets]
         not_modified = [x for x in ok
                         if x not in targets
@@ -1368,8 +1451,7 @@ def latest(
     '''
     rtag = __gen_rtag()
     refresh = bool(
-        salt.utils.is_true(refresh)
-        or (os.path.isfile(rtag) and refresh is not False)
+        salt.utils.is_true(refresh) or (os.path.isfile(rtag) and refresh is not False)
     )
 
     if kwargs.get('sources'):
@@ -1387,7 +1469,15 @@ def latest(
                     'comment': 'Invalidly formatted "pkgs" parameter. See '
                                'minion log.'}
     else:
-        desired_pkgs = [name]
+        if isinstance(pkgs, list) and len(pkgs) == 0:
+            return {
+                'name': name,
+                'changes': {},
+                'result': True,
+                'comment': 'No packages to install provided'
+            }
+        else:
+            desired_pkgs = [name]
 
     cur = __salt__['pkg.version'](*desired_pkgs, **kwargs)
     try:
@@ -1426,33 +1516,29 @@ def latest(
                 log.error(msg)
                 problems.append(msg)
             else:
-                if salt.utils.compare_versions(ver1=cur[pkg],
-                    oper='!=',
-                    ver2=avail[pkg],
-                    cmp_func=cmp_func):
+                if salt.utils.compare_versions(ver1=cur[pkg], oper='!=', ver2=avail[pkg], cmp_func=cmp_func):
                     targets[pkg] = avail[pkg]
                 else:
-                    if __salt__['portage_config.is_changed_uses'](pkg):
+                    if not cur[pkg] or __salt__['portage_config.is_changed_uses'](pkg):
                         targets[pkg] = avail[pkg]
     else:
         for pkg in desired_pkgs:
-            if not avail[pkg]:
-                if not cur[pkg]:
+            if pkg not in avail:
+                if not cur.get(pkg):
                     msg = 'No information found for \'{0}\'.'.format(pkg)
                     log.error(msg)
                     problems.append(msg)
-            elif not cur[pkg] \
-                    or salt.utils.compare_versions(ver1=cur[pkg],
-                                                   oper='<',
-                                                   ver2=avail[pkg],
-                                                   cmp_func=cmp_func):
+            elif not cur.get(pkg) \
+                    or salt.utils.compare_versions(ver1=cur[pkg], oper='<', ver2=avail[pkg], cmp_func=cmp_func):
                 targets[pkg] = avail[pkg]
 
     if problems:
-        return {'name': name,
-                'changes': {},
-                'result': False,
-                'comment': ' '.join(problems)}
+        return {
+            'name': name,
+            'changes': {},
+            'result': False,
+            'comment': ' '.join(problems)
+        }
 
     if targets:
         # Find up-to-date packages
@@ -1466,9 +1552,7 @@ def latest(
 
         if __opts__['test']:
             to_be_upgraded = ', '.join(sorted(targets))
-            comment = 'The following packages are set to be ' \
-                      'installed/upgraded: ' \
-                      '{0}'.format(to_be_upgraded)
+            comment = ['The following packages are set to be installed/upgraded: {0}'.format(to_be_upgraded)]
             if up_to_date:
                 up_to_date_nb = len(up_to_date)
                 if up_to_date_nb <= 10:
@@ -1477,19 +1561,16 @@ def latest(
                         '{0} ({1})'.format(name, cur[name])
                         for name in up_to_date_sorted
                     )
-                    comment += (
-                        ' The following packages are already '
-                        'up-to-date: {0}'
-                    ).format(up_to_date_details)
+                    comment.append('The following packages are already up-to-date: {0}'.format(up_to_date_details))
                 else:
-                    comment += ' {0} packages are already up-to-date'.format(
-                        up_to_date_nb
-                    )
+                    comment.append('{0} packages are already up-to-date'.format(up_to_date_nb))
 
-            return {'name': name,
-                    'changes': {},
-                    'result': None,
-                    'comment': comment}
+            return {
+                'name': name,
+                'changes': {},
+                'result': None,
+                'comment': ' '.join(comment)
+            }
 
         # Build updated list of pkgs to exclude non-targeted ones
         targeted_pkgs = list(targets.keys()) if pkgs else None
@@ -1586,6 +1667,7 @@ def _uninstall(
     version=None,
     pkgs=None,
     normalize=True,
+    ignore_epoch=False,
     **kwargs):
     '''
     Common function for package removal
@@ -1608,7 +1690,8 @@ def _uninstall(
                 'result': False,
                 'comment': 'An error was encountered while parsing targets: '
                            '{0}'.format(exc)}
-    targets = _find_remove_targets(name, version, pkgs, normalize, **kwargs)
+    targets = _find_remove_targets(name, version, pkgs, normalize,
+                                   ignore_epoch=ignore_epoch, **kwargs)
     if isinstance(targets, dict) and 'result' in targets:
         return targets
     elif not isinstance(targets, list):
@@ -1672,7 +1755,12 @@ def _uninstall(
             'comment': ' '.join(comments)}
 
 
-def removed(name, version=None, pkgs=None, normalize=True, **kwargs):
+def removed(name,
+            version=None,
+            pkgs=None,
+            normalize=True,
+            ignore_epoch=False,
+            **kwargs):
     '''
     Verify that a package is not installed, calling ``pkg.remove`` if necessary
     to remove the package.
@@ -1684,6 +1772,30 @@ def removed(name, version=None, pkgs=None, normalize=True, **kwargs):
         The version of the package that should be removed. Don't do anything if
         the package is installed with an unmatching version.
 
+        .. important::
+            As of version 2015.8.7, for distros which use yum/dnf, packages
+            which have a version with a nonzero epoch (that is, versions which
+            start with a number followed by a colon like in the example above)
+            must have the epoch included when specifying the version number.
+            For example:
+
+            .. code-block:: yaml
+
+                vim-enhanced:
+                  pkg.installed:
+                    - version: 2:7.4.160-1.el7
+
+            In version 2015.8.9, an **ignore_epoch** argument has been added to
+            :py:mod:`pkg.installed <salt.states.pkg.installed>`,
+            :py:mod:`pkg.removed <salt.states.pkg.installed>`, and
+            :py:mod:`pkg.purged <salt.states.pkg.installed>` states, which
+            causes the epoch to be disregarded when the state checks to see if
+            the desired version was installed. If **ignore_epoch** was not set
+            to ``True``, and instead of ``2:7.4.160-1.el7`` a version of
+            ``7.4.160-1.el7`` were used, this state would report success since
+            the actual installed version includes the epoch, and the specified
+            version would not match.
+
     normalize : True
         Normalize the package name by removing the architecture, if the
         architecture of the package is different from the architecture of the
@@ -1694,6 +1806,34 @@ def removed(name, version=None, pkgs=None, normalize=True, **kwargs):
 
         .. versionadded:: 2015.8.0
 
+    ignore_epoch : False
+        When a package version contains an non-zero epoch (e.g.
+        ``1:3.14.159-2.el7``, and a specific version of a package is desired,
+        set this option to ``True`` to ignore the epoch when comparing
+        versions. This allows for the following SLS to be used:
+
+        .. code-block:: yaml
+
+            # Actual vim-enhanced version: 2:7.4.160-1.el7
+            vim-enhanced:
+              pkg.removed:
+                - version: 7.4.160-1.el7
+                - ignore_epoch: True
+
+        Without this option set to ``True`` in the above example, the state
+        would falsely report success since the actual installed version is
+        ``2:7.4.160-1.el7``. Alternatively, this option can be left as
+        ``False`` and the full version string (with epoch) can be specified in
+        the SLS file:
+
+        .. code-block:: yaml
+
+            vim-enhanced:
+              pkg.removed:
+                - version: 2:7.4.160-1.el7
+
+        .. versionadded:: 2015.8.9
+
     Multiple Package Options:
 
     pkgs
@@ -1705,7 +1845,8 @@ def removed(name, version=None, pkgs=None, normalize=True, **kwargs):
     '''
     try:
         return _uninstall(action='remove', name=name, version=version,
-                          pkgs=pkgs, normalize=normalize, **kwargs)
+                          pkgs=pkgs, normalize=normalize,
+                          ignore_epoch=ignore_epoch, **kwargs)
     except CommandExecutionError as exc:
         return {'name': name,
                 'changes': {},
@@ -1713,7 +1854,12 @@ def removed(name, version=None, pkgs=None, normalize=True, **kwargs):
                 'comment': str(exc)}
 
 
-def purged(name, version=None, pkgs=None, normalize=True, **kwargs):
+def purged(name,
+           version=None,
+           pkgs=None,
+           normalize=True,
+           ignore_epoch=False,
+           **kwargs):
     '''
     Verify that a package is not installed, calling ``pkg.purge`` if necessary
     to purge the package. All configuration files are also removed.
@@ -1725,6 +1871,30 @@ def purged(name, version=None, pkgs=None, normalize=True, **kwargs):
         The version of the package that should be removed. Don't do anything if
         the package is installed with an unmatching version.
 
+        .. important::
+            As of version 2015.8.7, for distros which use yum/dnf, packages
+            which have a version with a nonzero epoch (that is, versions which
+            start with a number followed by a colon like in the example above)
+            must have the epoch included when specifying the version number.
+            For example:
+
+            .. code-block:: yaml
+
+                vim-enhanced:
+                  pkg.installed:
+                    - version: 2:7.4.160-1.el7
+
+            In version 2015.8.9, an **ignore_epoch** argument has been added to
+            :py:mod:`pkg.installed <salt.states.pkg.installed>`,
+            :py:mod:`pkg.removed <salt.states.pkg.installed>`, and
+            :py:mod:`pkg.purged <salt.states.pkg.installed>` states, which
+            causes the epoch to be disregarded when the state checks to see if
+            the desired version was installed. If **ignore_epoch** was not set
+            to ``True``, and instead of ``2:7.4.160-1.el7`` a version of
+            ``7.4.160-1.el7`` were used, this state would report success since
+            the actual installed version includes the epoch, and the specified
+            version would not match.
+
     normalize : True
         Normalize the package name by removing the architecture, if the
         architecture of the package is different from the architecture of the
@@ -1735,6 +1905,34 @@ def purged(name, version=None, pkgs=None, normalize=True, **kwargs):
 
         .. versionadded:: 2015.8.0
 
+    ignore_epoch : False
+        When a package version contains an non-zero epoch (e.g.
+        ``1:3.14.159-2.el7``, and a specific version of a package is desired,
+        set this option to ``True`` to ignore the epoch when comparing
+        versions. This allows for the following SLS to be used:
+
+        .. code-block:: yaml
+
+            # Actual vim-enhanced version: 2:7.4.160-1.el7
+            vim-enhanced:
+              pkg.purged:
+                - version: 7.4.160-1.el7
+                - ignore_epoch: True
+
+        Without this option set to ``True`` in the above example, the state
+        would falsely report success since the actual installed version is
+        ``2:7.4.160-1.el7``. Alternatively, this option can be left as
+        ``False`` and the full version string (with epoch) can be specified in
+        the SLS file:
+
+        .. code-block:: yaml
+
+            vim-enhanced:
+              pkg.purged:
+                - version: 2:7.4.160-1.el7
+
+        .. versionadded:: 2015.8.9
+
     Multiple Package Options:
 
     pkgs
@@ -1746,7 +1944,8 @@ def purged(name, version=None, pkgs=None, normalize=True, **kwargs):
     '''
     try:
         return _uninstall(action='purge', name=name, version=version,
-                          pkgs=pkgs, normalize=normalize, **kwargs)
+                          pkgs=pkgs, normalize=normalize,
+                          ignore_epoch=ignore_epoch, **kwargs)
     except CommandExecutionError as exc:
         return {'name': name,
                 'changes': {},
@@ -1804,12 +2003,10 @@ def uptodate(name, refresh=False, **kwargs):
 
     if updated.get('result') is False:
         ret.update(updated)
-    elif updated:
-        ret['changes'] = updated
-        ret['comment'] = 'Upgrade successful.'
-        ret['result'] = True
     else:
-        ret['comment'] = 'Upgrade failed.'
+        ret['changes'] = updated.get('changes', {})
+        ret['comment'] = 'Upgrade ran successfully.'
+        ret['result'] = True
 
     return ret
 
@@ -1818,8 +2015,8 @@ def group_installed(name, skip=None, include=None, **kwargs):
     '''
     .. versionadded:: 2015.8.0
 
-    Ensure that an entire package group is installed. This state is only
-    supported for the :mod:`yum <salt.modules.yumpkg>` package manager.
+    Ensure that an entire package group is installed. This state is currently
+    only supported for the :mod:`yum <salt.modules.yumpkg>` package manager.
 
     skip
         Packages that would normally be installed by the package group
@@ -1837,7 +2034,6 @@ def group_installed(name, skip=None, include=None, **kwargs):
         installed by a ``yum groupinstall`` ("optional" packages). Note that
         this will not enforce group membership; if you include packages which
         are not members of the specified groups, they will still be installed.
-        Can be passed either as a comma-separated list or a python list.
 
         .. code-block:: yaml
 
@@ -1846,12 +2042,15 @@ def group_installed(name, skip=None, include=None, **kwargs):
                 - include:
                   - haproxy
 
-    .. note::
+        .. versionchanged:: Boron
+            This option can no longer be passed as a comma-separated list, it
+            must now be passed as a list (as shown in the above example).
 
+    .. note::
         Because this is essentially a wrapper around :py:func:`pkg.install
         <salt.modules.yumpkg.install>`, any argument which can be passed to
-        pkg.install may also be included here, and it will be passed along
-        wholesale.
+        pkg.install may also be included here, and it will be passed on to the
+        call to :py:func:`pkg.install <salt.modules.yumpkg.install>`.
     '''
     ret = {'name': name,
            'changes': {},
@@ -1859,47 +2058,32 @@ def group_installed(name, skip=None, include=None, **kwargs):
            'comment': ''}
 
     if 'pkg.group_diff' not in __salt__:
-        ret['comment'] = 'pkg.group_install not implemented for this platform'
+        ret['comment'] = 'pkg.group_install not available for this platform'
         return ret
 
-    if skip is not None:
-        if isinstance(skip, six.string_types):
-            skip = skip.split(',')
-        elif isinstance(skip, (float, six.integer_types)):
-            skip = [str(skip)]
+    if skip is None:
+        skip = []
+    else:
         if not isinstance(skip, list):
             ret['comment'] = 'skip must be formatted as a list'
             return ret
         for idx, item in enumerate(skip):
-            if isinstance(item, (float, six.integer_types)):
+            if not isinstance(item, six.string_types):
                 skip[idx] = str(item)
-            if not isinstance(skip[idx], six.string_types):
-                ret['comment'] = 'Invalid \'skip\' item {0}'.format(skip[idx])
-                return ret
-    else:
-        skip = []
 
-    if include is not None:
-        if isinstance(include, six.string_types):
-            include = include.split(',')
-        elif isinstance(include, (float, six.integer_types)):
-            include = [str(include)]
+    if include is None:
+        include = []
+    else:
         if not isinstance(include, list):
             ret['comment'] = 'include must be formatted as a list'
             return ret
         for idx, item in enumerate(include):
-            if isinstance(item, (float, six.integer_types)):
+            if not isinstance(item, six.string_types):
                 include[idx] = str(item)
-            if not isinstance(include[idx], six.string_types):
-                ret['comment'] = \
-                    'Invalid \'include\' item {0}'.format(include[idx])
-                return ret
-    else:
-        include = []
 
     diff = __salt__['pkg.group_diff'](name)
-    mandatory = diff['mandatory packages']['installed'] + \
-        diff['mandatory packages']['not installed']
+    mandatory = diff['mandatory']['installed'] + \
+        diff['mandatory']['not installed']
 
     invalid_skip = [x for x in mandatory if x in skip]
     if invalid_skip:
@@ -1909,8 +2093,8 @@ def group_installed(name, skip=None, include=None, **kwargs):
         )
         return ret
 
-    targets = diff['mandatory packages']['not installed']
-    targets.extend([x for x in diff['default packages']['not installed']
+    targets = diff['mandatory']['not installed']
+    targets.extend([x for x in diff['default']['not installed']
                     if x not in skip])
     targets.extend(include)
 
@@ -1919,9 +2103,9 @@ def group_installed(name, skip=None, include=None, **kwargs):
         ret['comment'] = 'Group \'{0}\' is already installed'.format(name)
         return ret
 
-    partially_installed = diff['mandatory packages']['installed'] \
-        or diff['default packages']['installed'] \
-        or diff['optional packages']['installed']
+    partially_installed = diff['mandatory']['installed'] \
+        or diff['default']['installed'] \
+        or diff['optional']['installed']
 
     if __opts__['test']:
         ret['result'] = None
